# Project Based Tutorials in C

A list of tutorials that work towards the making of small to large projects in C.

## Table of Contents

* [Computer Networking](#computer-networking)
* [Databases](#databases)
* [Game Development](#game-development)
* [Operating Systems](#operating-systems)
* [Programming Languages](#programming-languages)
* [Uncategorized](#uncategorized)
* [Other Resources](#other-resources)

## Computer Networking

* [Beej's Guide to Network Programming](http://beej.us/guide/bgnet/html/multi/index.html) `book`
* [Learn Socket Programming in C from Scratch](https://www.udemy.com/learn-socket-programming-in-c-from-scratch/) `video`
* [Let's code a TCP/IP stack](http://www.saminiir.com/lets-code-tcp-ip-stack-1-ethernet-arp/)
* [Let's make a NTP Client in C](https://lettier.github.io/posts/2016-04-26-lets-make-a-ntp-client-in-c.html)
* [Multiplayer WebSocket Game server written in C using libuv & libwebsockets & Cocos2d-x-HTML5](http://www.gamedevcraft.com/2016/08/part-1-multiplayer-websocket-game.html)

## Databases

* [Let's Build a Simple Database](https://cstack.github.io/db_tutorial/)

## Game Development

* [Astroids Clone](https://dashgl.com/Astroids/)
* [Brickout Clone](https://dashgl.com/Brickout/)
* [Chess Engine In C](https://www.youtube.com/playlist?list=PLZ1QII7yudbc-Ky058TEaOstZHVbT-2hg) `video`
* [Coding A Sudoku Solver in C](https://www.youtube.com/playlist?list=PLkTXsX7igf8edTYU92nU-f5Ntzuf-RKvW) `video`
* [Coding a Rogue/Nethack RPG in C](https://www.youtube.com/playlist?list=PLkTXsX7igf8erbWGYT4iSAhpnJLJ0Nk5G) `video`
* [Game Engine Black Book: Wolfenstein 3D](https://www.amazon.com/dp/1539692876/) `book`
* [Handmade Hero](https://handmadehero.org/) `video` `in-progress`
* [Handmade Quake](https://www.youtube.com/playlist?list=PLBKDuv-qJpTbCsXHsxcoSSsMarnfyNhHF) `video` `in-progress`
* [Hangman](https://www.youtube.com/playlist?list=PLZ1QII7yudbd2ZHYSEWrSddsvD5PW_r5O) `video`
* [Invaders Clone](https://dashgl.com/Invaders/)
* [How to Program an NES game in C](https://nesdoug.com/)
* [How to Program a Text Adventure in C](https://helderman.github.io/htpataic/htpataic01.html) `in-progress`
* [Learn Video Game Programming in C](https://www.youtube.com/playlist?list=PLT6WFYYZE6uLMcPGS3qfpYm7T_gViYMMt) `video`
* [Let's Make: Dangerous Dave](https://www.youtube.com/playlist?list=PLSkJey49cOgTSj465v2KbLZ7LMn10bCF9) `video`
* [Making 8-bit Arcade Games in C](https://www.amazon.com/dp/1545484759/) `book`
* [Making a Text Adventure Game in C](https://www.youtube.com/watch?v=sLcrmZyWgaM) `video` `in-progress`
* [Simple Tic Tac Toe in C](https://www.youtube.com/playlist?list=PLZ1QII7yudbc7_ZgXA-gIXmME41Rs2GP5) `video`
* [Sudoku Programming with C](https://www.amazon.com/dp/1484209966/) `book`
* [Tic-tac-toe Game in C with SDL](https://www.youtube.com/watch?v=gCVMkKgs3uQ) `video`

## Operating Systems

* [Bitwise](https://github.com/pervognsen/bitwise) `video` `in-progress`
* [Building an Operating System for the Raspberry Pi](https://jsandler18.github.io/)
* [Hack the Virtual Memory](https://blog.holbertonschool.com/hack-virtual-memory-stack-registers-assembly-code/)
* [How to Create an OS from Scratch](https://github.com/cfenollosa/os-tutorial) `in-progress`
* [Learning operating system development using Linux kernel and Raspberry Pi](https://github.com/s-matyukevich/raspberry-pi-os) `in-progress`
* [Let's Write a Malloc](https://danluu.com/malloc-tutorial/)
* Let’s write a Kernel
    * [Part 1 - Kernel 101](https://arjunsreedharan.org/post/82710718100/kernel-101-lets-write-a-kernel)
    * [Part 2 - Kernel 201](https://arjunsreedharan.org/post/99370248137/kernel-201-lets-write-a-kernel-with-keyboard)
* [Operating Systems Design and Implementation](https://www.amazon.com/dp/0131429388) `book`
* [Operating Systems: From 0 to 1](https://tuhdo.github.io/os01/) `book`
* [The little book about OS development](https://littleosbook.github.io/) `book`
* [Write a Shell in C](https://brennan.io/2015/01/16/write-a-shell-in-c/)
* [Write a Simple Memory Allocator](https://arjunsreedharan.org/post/148675821737/write-a-simple-memory-allocator)
* [Write a System Call](https://brennan.io/2016/11/14/kernel-dev-ep3/)
* [Writing a FUSE Filesystem](https://www.cs.nmsu.edu/~pfeiffer/fuse-tutorial/)
* Writing a Unix Shell
   * [Part 1](https://indradhanush.github.io/blog/writing-a-unix-shell-part-1)
   * [Part 2](https://indradhanush.github.io/blog/writing-a-unix-shell-part-2)
   * [Part 3](https://indradhanush.github.io/blog/writing-a-unix-shell-part-3)

## Programming Languages

* [A Retargetable C Compiler: Design and Implementation](https://www.amazon.com/dp/0805316701/) `book`
* [Baby's First Garbage Collector](http://journal.stuffwithstuff.com/2013/12/08/babys-first-garbage-collector/)
* [Build Your Own Lisp](http://www.buildyourownlisp.com/) `book`
* [Compiler Design in C](https://www.amazon.com/dp/0131550454) `book`
* [Crafting Interpreters](http://www.craftinginterpreters.com/) `book` `in-progress`
* [Implementing a Virtual Machine in C](https://felixangell.com/blog/virtual-machine-in-c)
* [Let's Build a Compiler: A C & x86 version](https://github.com/lotabout/Let-s-build-a-compiler)
* [Modern Compiler Implementation in C](https://www.amazon.com/dp/052158390X/) `book`
* Write a C Interpreter
    * [Part 0 - Preface](https://github.com/lotabout/write-a-C-interpreter/blob/master/tutorial/en/0-Preface.md)
    * [Part 1 - Skeleton](https://github.com/lotabout/write-a-C-interpreter/blob/master/tutorial/en/1-Skeleton.md)
    * [Part 2 - Virtual Machine](https://github.com/lotabout/write-a-C-interpreter/blob/master/tutorial/en/2-Virtual-Machine.md)
    * [Part 3 - Lexer](https://github.com/lotabout/write-a-C-interpreter/blob/master/tutorial/en/3-Lexer.md)
    * [Part 4 - Top-down Parsing](https://github.com/lotabout/write-a-C-interpreter/blob/master/tutorial/en/4-Top-down-Parsing.md)
    * [Part 5 - Variables](https://github.com/lotabout/write-a-C-interpreter/blob/master/tutorial/en/5-Variables.md)
    * [Part 6 - Functions](https://github.com/lotabout/write-a-C-interpreter/blob/master/tutorial/en/6-Functions.md)
    * [Part 7 - Statements](https://github.com/lotabout/write-a-C-interpreter/blob/master/tutorial/en/7-Statements.md)
    * [Part 8 - Expressions](https://github.com/lotabout/write-a-C-interpreter/blob/master/tutorial/en/8-Expressions.md)
* [Writing a Simple Garbage Collector in C](http://maplant.com/gc.html)
* Scheme from Scratch
    * [Part 1 - Introduction](http://peter.michaux.ca/articles/scheme-from-scratch-introduction)
    * [Part 2 - Integers](http://peter.michaux.ca/articles/scheme-from-scratch-bootstrap-v0_1-integers)
    * [Part 3 - Booleans](http://peter.michaux.ca/articles/scheme-from-scratch-bootstrap-v0_2-booleans)
    * [Part 4 - Characters](http://peter.michaux.ca/articles/scheme-from-scratch-bootstrap-v0_3-characters)
    * [Part 5 - Strings](http://peter.michaux.ca/articles/scheme-from-scratch-bootstrap-v0_4-strings)
    * [Part 6 - The Empty List](http://peter.michaux.ca/articles/scheme-from-scratch-bootstrap-v0_5-the-empty-list)
    * [Part 7 - Pairs](http://peter.michaux.ca/articles/scheme-from-scratch-bootstrap-v0_6-pairs)
    * [Part 8 - Symbols](http://peter.michaux.ca/articles/scheme-from-scratch-bootstrap-v0_7-symbols)
    * [Part 9 - Quote](http://peter.michaux.ca/articles/scheme-from-scratch-bootstrap-v0_8-quote)
    * [Part 10 - Environments](http://peter.michaux.ca/articles/scheme-from-scratch-bootstrap-v0_9-environments)
    * [Part 11 - if](http://peter.michaux.ca/articles/scheme-from-scratch-bootstrap-v0_10-if)
    * [Part 12 - Primitive Procedures Part 1](http://peter.michaux.ca/articles/scheme-from-scratch-bootstrap-v0_11-primitive-procedures-part-1)
    * [Part 13 - Primitive Procedures Part 2](http://peter.michaux.ca/articles/scheme-from-scratch-bootstrap-v0_12-primitive-procedures-part-2)
    * [Part 14 - Lambda the Ultimate](http://peter.michaux.ca/articles/scheme-from-scratch-bootstrap-v0_13-lambda-the-ultimate)
    * [Part 15 - Begin](http://peter.michaux.ca/articles/scheme-from-scratch-bootstrap-v0_14-begin)
    * [Part 16 - Cond](http://peter.michaux.ca/articles/scheme-from-scratch-bootstrap-v0_15-cond)
    * [Part 17 - Let](http://peter.michaux.ca/articles/scheme-from-scratch-bootstrap-v0_16-let)
    * [Part 18 - And and Or](http://peter.michaux.ca/articles/scheme-from-scratch-bootstrap-v0_17-and-and-or)
    * [Part 19 - Apply](http://peter.michaux.ca/articles/scheme-from-scratch-bootstrap-v0_18-apply)
    * [Part 20 - Eval](http://peter.michaux.ca/articles/scheme-from-scratch-bootstrap-v0_19-eval)
    * [Part 21 - I/O](http://peter.michaux.ca/articles/scheme-from-scratch-bootstrap-v0_20-io)
    * [Part 22 - Standard Library](http://peter.michaux.ca/articles/scheme-from-scratch-bootstrap-v0_21-standard-library)
    * [Part 23 - Garbage Collection](http://peter.michaux.ca/articles/scheme-from-scratch-bootstrap-v0_22-garbage-collection)
    * [Part 24 - Conclusion](http://peter.michaux.ca/articles/scheme-from-scratch-bootstrap-conclusion)

## Uncategorized

* [A Regular Expression Matcher](https://www.cs.princeton.edu/courses/archive/spr09/cos333/beautiful.html)
* [Build Your Own Text Editor](https://viewsourcecode.org/snaptoken/kilo/)
* [Emulator 101](http://emulator101.com/)
* [How to Write a Video Player in Less Than 1000 Lines](http://dranger.com/ffmpeg/ffmpeg.html)
* [Learn FFmpeg libav the Hard Way](https://github.com/leandromoreira/ffmpeg-libav-tutorial) `in-progress`
* [Linux Containers in 500 Lines of Code](https://blog.lizzie.io/linux-containers-in-500-loc.html)
* [Making a Heap Allocator](https://handmade.network/wiki/2877-tutorial_making_a_heap_allocator)
* [The Audio Programming Book](https://www.amazon.com/Audio-Programming-Book-MIT-Press/dp/0262014467/) `book`
* [The Craft of Text Editing](https://www.amazon.com/dp/1411682971/) `book`
* [Write a hash table in C](https://github.com/jamesroutley/write-a-hash-table)
* [Writing an SVG Library](http://www.code-in-c.com/writing-svg-library-c/)

## Other Resources

* Practice Projects
    * [Beginner Projects](https://github.com/jorgegonzalez/beginner-projects)
    * [Mega Project List](https://github.com/karan/Projects)
    * [The UChicago χ-Projects](http://chi.cs.uchicago.edu/index.html)
* Similar List  
    * [awesome-diy-software](https://github.com/cweagans/awesome-diy-software)
    * [Build Your Own X](https://github.com/danistefanovic/build-your-own-x)
    * [Projects-from-Scratch](https://github.com/AlgoryL/Projects-from-Scratch)
    * [Project Based Learning](https://github.com/tuvtran/project-based-learning)
* C
    * [awesome-c](https://github.com/kozross/awesome-c)
    * [Modern C](http://icube-icps.unistra.fr/img_auth.php/d/db/ModernC.pdf)
